#ifndef INCLUDED_CURVE_EDITOR_H
#define INCLUDED_CURVE_EDITOR_H

#pragma warning(disable : 4786)

#include <string>

#include "Zoomer.h"

using namespace std;

class CurveEvaluator;

class Curve;

class CurveEditor
{
	public:

		CurveEditor(void);

		~CurveEditor(void);

		void resetCurveEditor(void);

		void specifyWindowSize(const unsigned int & window_width, const unsigned int & window_height);

		void changeAnimationLength(const double & animation_length);

		void rescaleTime(const double & scale_factor);

		double getMinimumTime(void) const;

		double getMaximumTime(void) const;

		double getMinimumValue(void) const;

		double getMaximumValue(void) const;

		void drawCurveEditor(const double & current_time) const;

		void mouseDown(const int & x_location, const int & y_location);

		void altClick(const int & x_location, const int & y_location);

		void controlClick(const int & x_location, const int & y_location);

		void mouseMove(const int & x_location, const int & y_location);

		void mouseUp(const int & x_location, const int & y_location);

		void shiftMouseDown(const int & x_location, const int & y_location);

		void shiftMouseDrag(const int & x_location, const int & y_location);

		void shiftMouseUp(const int & x_location, const int & y_location);

		void tryPopZoom(void);

		void zoomOutTotally(void);

		void changeMinimumY(const double & new_minimum_y);
		
		void changeMaximumY(const double & new_maximum_y);

		Curve * getCurrentlySelectedCurve(void) const;

		void toggleShowAnimationCurve(const string & name);

		void toggleWrapControlPoints(void);

		double getTime(const unsigned int & x_location) const;
		
		double getNormalizedX(const unsigned int & x_location) const;
		double getNormalizedY(const unsigned int & y_location) const;

		void findNiceDivisions(const double & minimum, const double & maximum, double * new_start, double * new_division_interval, double * new_end) const;

	private:

		static const double CONTROL_POINT_SELECTION_TOLERANCE;
		static const double CURVE_SELECTION_TOLERANCE;

		static const float GRID_BACKGROUND_INTENSITY;
		static const float GRID_LINE_INTENSITY;
		static const float GRID_SHADOW_INTENSITY;

		static const unsigned int CurveEditor::NUMBER_OF_TICKS;

		void setupWindow(void) const;

		void drawAllCurves(void) const;

		void drawHashMarks(void) const;

		void drawIncompleteZoom(void) const;

		void drawTimeBar(const double & current_time) const;

		int clamp(const int & x, const int & a, const int & b) const;

		void validateCoordinates(const int & x_location, const int & y_location) const;

		unsigned int m_window_width;
		unsigned int m_window_height;

		ZoomDimensions m_incomplete_zoom;

		bool m_have_incomplete_zoom;

		Curve * m_currently_selected_curve;

		unsigned int m_currently_curve_moving_control_point_index;

		bool m_currently_curve_moving_control_point_index_is_valid;

		bool m_wrap_control_points;
};

#endif