//
// scene.h
//
// The Scene class
//

#ifndef __SCENE_H__
#define __SCENE_H__

#pragma warning(disable:4786)

#include <list>
#include <algorithm>

using namespace std;

#include <Fl/Fl_Window.H>
#include <Fl/Fl_Button.H>
#include <Fl/Fl_Light_Button.H>
#include <Fl/Fl_Check_Button.H>
#include <Fl/Fl_Box.H>
#include <Fl/Fl_Group.H>

#include "object.h"

class CAniDoc;
class CGLView;

class scene_params {
public:
	scene_params(void *p, Object *o) { self=p, obj=o; };
	void *self;
	Object* obj;
};

class Scene {
public:
	typedef list<Object*>::iterator			giter;
	typedef list<Object*>::const_iterator	cgiter;

public:
	Scene() : objects() { m_nObjCnt=0; m_pView=NULL; m_pObjList=NULL; m_pActiveObj=NULL; };
	Scene(Object* obj);
	~Scene();

	void add( Object* obj )
	{ objects.push_back( obj ); m_nObjCnt++; obj->setScene(this); }
	
	void redraw(int drawMode);

	void refresh();

	void setDocument(CAniDoc* pDoc) { m_pDoc=pDoc;};
	CAniDoc *getDocument() { return m_pDoc; };

	void draw_mode_setup(int drawMode);
	void draw_mode_finish(int drawMode);

	void show();

	void setView(CGLView *view) { m_pView=view; };
	CGLView *getView() { return m_pView; };

	void createView();
	void createObjList();

	void saveImage(char *iname);

	Fl_Window *m_pObjList;

	Object *m_pActiveObj;

private:
    list<Object*> objects;
	CGLView *m_pView;

	int m_nObjCnt;
	CAniDoc *m_pDoc;

	static void cb_fork_object(Fl_Widget* o, void* v);
	static void cb_toggle_object(Fl_Widget* o, void* v);
	static void cb_activate_object(Fl_Widget* o, void* v);
};

#endif